/**
 * This form displays all the conflicts that occurred and asks the user
 * how they wish to resolve the conflict.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using HR;
using LOpen.EntityModel;

namespace ChangeConflictResolution
{
    public partial class ConflictForm : Form
    {
        private readonly ObjectChangeConflict conflict;

        public ConflictForm(ApplyChangesException ex)
        {
            InitializeComponent();

            // 
            conflict = ex.ChangeConflicts[0];

            //
            int top = 0;

            // Iterate through all the conflicting fields
            foreach (MemberChangeConflict mcc in conflict.MemberConflicts)
            {
				// Create a label for the field's name
                Label labelField = new Label();
                labelField.Text = mcc.Column.Label;
                labelField.Top = top;
                panelFields.Controls.Add(labelField);

				// Create a label for the field's current in-memory value
                Label labelMyValue = new Label();
                labelMyValue.Top = top;
                labelMyValue.Text = mcc.CurrentValue.ToString();
                panelMyValues.Controls.Add(labelMyValue);

				// Create a label for the field's current value in the database
                Label labelDbValue = new Label();
                labelDbValue.Top = top;
                labelDbValue.Text = mcc.DatabaseValue.ToString();
                panelDbValues.Controls.Add(labelDbValue);

				// Create a panel to host the buttons 
				// ('Keep Mine' & 'Keep Database')
                Panel buttonPanel = new Panel();
                buttonPanel.Left = 0;
                buttonPanel.Width = panelActions.Width;
                buttonPanel.Top = top;

                Button btnKeepMyValue = new Button();
                btnKeepMyValue.AutoSize = true;
                btnKeepMyValue.Text = "Keep Mine";
                btnKeepMyValue.Tag = mcc;
                btnKeepMyValue.Click += new EventHandler(btnKeepMyValue_Click);
                buttonPanel.Controls.Add(btnKeepMyValue);

                Button btnKeepDbValue = new Button();
                btnKeepDbValue.Text = "Keep Database";
                btnKeepDbValue.AutoSize = true;
                btnKeepDbValue.Left = btnKeepMyValue.Right + 5;
                btnKeepDbValue.Tag = mcc;
                btnKeepDbValue.Click += new EventHandler(btnKeepDbValue_Click);
                buttonPanel.Controls.Add(btnKeepDbValue);

                buttonPanel.Height = btnKeepDbValue.Height;
                panelActions.Controls.Add(buttonPanel);

                //
                top += 25;
            }
        }

		/// <summary>
		/// Keep database value
		/// </summary>
        void btnKeepDbValue_Click(object sender, EventArgs e)
        {
            Button btn = (Button)sender;
            MemberChangeConflict memberInfo = (MemberChangeConflict)btn.Tag;

			// Resolve the conflict - overwrite current in-memory value
            memberInfo.Resolve(ConflictResolveMode.OverwriteCurrentValues);

			//
            btn.Font = new Font(btn.Font, FontStyle.Bold);
            foreach (Control c in btn.Parent.Controls) c.Enabled = false;
        }

		/// <summary>
		/// Keep my current in-memory value
		/// </summary>
        void btnKeepMyValue_Click(object sender, EventArgs e)
        {
            Button btn = (Button)sender;
            MemberChangeConflict memberInfo = (MemberChangeConflict)btn.Tag;

			// Resolve the conflict - keep my current in-memory value
            memberInfo.Resolve(ConflictResolveMode.KeepCurrentValues);

			//
            btn.Font = new Font(btn.Font, FontStyle.Bold);
            foreach (Control c in btn.Parent.Controls) c.Enabled = false;
        }

		/// <summary>
		/// Keep my values for all fields
		/// </summary>
        private void btnKeepAllMyValues_Click(object sender, EventArgs e)
        {
            conflict.Resolve(ConflictResolveMode.KeepCurrentValues);
        }

		/// <summary>
		/// Keep database values for all fields
		/// </summary>
        private void btnKeepAllDbValues_Click(object sender, EventArgs e)
        {
            conflict.Resolve(ConflictResolveMode.OverwriteCurrentValues);
        }
    }
}