﻿/**
 * This sample illustrates how to send an email message on an IBMi system.
 **/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using LOpen.EntityModel;
using Util;
using Model;

namespace SendEmail
{
	public partial class MainForm : Form
	{
		private readonly DataContext context;

		public MainForm(ConnectionForm cf)
		{
			InitializeComponent();

			//
			context = new DataContext(false);
			context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);
		}

		// Send the message
		private void btnSend_Click(object sender, EventArgs e)
		{
			// Create and setup an email message object
			var message = new IbmMailMessage();

			message.Sender = txtFrom.Text;
			message.Subject = txtSubject.Text;
			message.Body = txtBody.Text;

			// Parse target and cc addresses & add the the respective lists
			ParseAndAddAddresses(txtTo.Text, message.ToList);
			ParseAndAddAddresses(txtCc.Text, message.CCList);

			//
			try
			{
				// Send the email
				context.Ibm.SendMail(message);
				MessageBox.Show("Message has been sent successfully.", "OK");
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Error");
			}
		}

		// Parse email addresses (separated by comma or semi-colon
		// and add to the specified address list in an email message.
		private void ParseAndAddAddresses(string addresses, IbmMailAddressList addressList)
		{
			char? separator = null;
			addresses = addresses.Trim();

			//
			if (addresses.Length > 0)
			{
				if (addresses.Contains(",")) separator = ',';
				else if (addresses.Contains(";")) separator = ';';

				//
				string[] addrArray = { addresses };
				if (separator != null)
				{
					addrArray = addresses.Split(separator.Value);
				}

				//
				foreach (string addr in addrArray) addressList.Add(addr);
			}
		}
	}
}
