﻿/**
 * This sample illustrates how to get list of spool files
 * and retrieve their contents.
 * 
 * Key methods:
 *   DataContext.Ibm.GetSpoolFileListForUser
 *   DataContext.Ibm.GetSpoolFileListForQueue
 **/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Util;
using Model;
using LOpen.EntityModel;

namespace SpoolFiles
{
	public partial class MainForm : Form
	{
		private readonly DataContext context;

		public MainForm(ConnectionForm cf)
		{
			InitializeComponent();
			cbType.SelectedIndex = 0;

			//
			context = new DataContext(false);
			context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);
		}

		private void btnGenerateList_Click(object sender, EventArgs e)
		{
			lblStatus.Text = "Retrieving spool files ...";
			statusStrip.Refresh();
			try
			{
				SpoolFile[] spoolFiles;
				string name = txtName.Text.Trim().ToUpperInvariant();

				//
				if (cbType.SelectedIndex == 0)
				{
					// Get spool file list for the specified user
					spoolFiles = context.Ibm.GetSpoolFileListForUser(name);
				}
				else
				{
					// Get spool file list for the specified queue
					spoolFiles = context.Ibm.GetSpoolFileListForQueue(name);
				}

				//
				foreach (var spoolFile in spoolFiles)
				{
					ListViewItem item = new ListViewItem();
					item.Text = spoolFile.Name;
					item.SubItems.Add(spoolFile.UserName);
					item.SubItems.Add(spoolFile.Queue);
					item.SubItems.Add(spoolFile.JobName);
					item.SubItems.Add(spoolFile.JobNumber.ToString());
					item.SubItems.Add(spoolFile.Library);
					item.SubItems.Add(spoolFile.PageCount.ToString());
					item.SubItems.Add(spoolFile.CurrentPage.ToString());
					item.SubItems.Add(spoolFile.Priority);
					item.Tag = spoolFile;

					//
					spoolFileList.Items.Add(item);
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Error");
			}
			finally
			{
				lblStatus.Text = "";
			}
		}

		private void ViewSpoolFileContent()
		{
			if (spoolFileList.SelectedItems.Count > 0)
			{
				SpoolFile spoolFile = spoolFileList.SelectedItems[0].Tag as SpoolFile;
				if (spoolFile != null)
				{
					lblStatus.Text = "Retrieving content ...";
					statusStrip.Refresh();
					try
					{
						txtContent.Text = spoolFile.Content;
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.Message, "Error");
					}
					finally
					{
						lblStatus.Text = "";
					}
				}
			}
			else
			{
				MessageBox.Show("Please select a spool file to view", "Error");
			}
		}

		private void btnViewContent_Click(object sender, EventArgs e)
		{
			ViewSpoolFileContent();
		}
	}
}
