﻿/**
 * This sample illustrates how to upload and download file from the server.
 * 
 * Key methods:
 *   DataContext.Services.TransferFileToServer
 *   DataContext.Services.TransferFileToClient
 **/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using Model;
using Util;
using LOpen.EntityModel;

namespace UploadDownloadFile
{
	public partial class MainForm : Form
	{
		private readonly DataContext context;

		public MainForm(ConnectionForm cf)
		{
			InitializeComponent();

			context = new DataContext(false);
			context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);
		}

		private void btnChooseFileToUpload_Click(object sender, EventArgs e)
		{
			if (this.openFileDialog.ShowDialog() == DialogResult.OK)
			{
				this.txtFileToUpload.Text = this.openFileDialog.FileName;
			}
		}

		private void btnChooseTargetFolder_Click(object sender, EventArgs e)
		{
			if (this.folderBrowserDialog.ShowDialog() == DialogResult.OK)
			{
				this.txtTargetFolder.Text = this.folderBrowserDialog.SelectedPath;
			}
		}

		private void btnUpload_Click(object sender, EventArgs e)
		{
			string sourcePath = this.txtFileToUpload.Text;
			string targetPath = this.txtUploadTo.Text.Trim();

			//
			string fileName = Path.GetFileName(sourcePath);
			if (!targetPath.EndsWith("/") && !targetPath.EndsWith(@"\")) targetPath += "/";
			targetPath += fileName;

			//
			if (sourcePath.Length > 0)
			{
				try
				{
					context.Services.TransferFileToServer(sourcePath, targetPath, FileTransferMode.Binary);
					MessageBox.Show("The file has been uploaded successfully.", "OK");
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message, "Error");
				}
			}
			else
			{
				MessageBox.Show("Please choose file to upload", "Error");
			}
		}

		private void btnDownload_Click(object sender, EventArgs e)
		{
			string folder = this.txtTargetFolder.Text;
			if (folder.Length > 0)
			{
				string sourcePath = this.txtFileToDownload.Text;
				string targetPath = this.txtTargetFolder.Text;

				//
				string fileName = Path.GetFileName(sourcePath);
				targetPath = Path.Combine(targetPath, fileName);

				//
				if (sourcePath.Length > 0 && targetPath.Length > 0)
				{
					try
					{
						context.Services.TransferFileToClient(sourcePath, targetPath, FileTransferMode.Binary);
						MessageBox.Show("The file has been downloaded successfully.", "OK");
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.Message, "Error");
					}
				}
				else
				{
					MessageBox.Show("Please choose file to download & target folder", "Error");
				}
			}
			else
			{
				MessageBox.Show("Please choose target folder to download the file to", "Error");
			}
		}
	}
}
