﻿Public Class MainForm
    Private masterContext As DataContext
    Private departments As Department()

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Create a master context (non-updatable)
        ' Used to retrieve readonly data such as department list/employee list
        masterContext = New DataContext(False)

        Using cf = New Util.ConnectionForm
            cf.ShowDialog()
            masterContext.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password)
        End Using

        ' Populate the events table with some data for the purposes of this example.
        PopulateEventsTable()

        ' Populate the department list from the DEPTAB table.
        ' Query the database table
        departments = masterContext.Departments.RetrieveList()

        ' Populate the combobox
        Dim index As Integer
        For Each department As Department In departments
            index = cmbDepartment.Items.Add(department.Description)
        Next

        cmbDepartment.SelectedIndex = 0

    End Sub

    Private Sub pubGetEvents_Click(ByVal sender As Object, ByVal e As EventArgs) Handles pubGetEvents.Click
        lvwEmployeeTotals.Items.Clear()
        lvwEvents.Items.Clear()

        Dim info = masterContext.GetEmployeeEvents.CreateInvokeInfo()

        ' Add a dummy record to show that the list contents are passed to the server
        Dim empEventItem = info.Lists.EmployeeEvents.NewItem()
        empEventItem.EmployeeNumber = "Dummy"
        empEventItem.EmployeeEventCount = 99999
        info.Lists.EmployeeEvents.Add(empEventItem)

        ' Call the server function passing the department whose events are to be returned.
        info.[In].Department = departments(cmbDepartment.SelectedIndex).Id
        masterContext.GetEmployeeEvents.Invoke(info)

        ' Move the values from the Working List to the listView control using an indexed
        ' for loop. The field names in the EmployeeEvents working list have been renamed
        ' in the LANSA Model to more meaningful names.
        Dim listData As [String]() = New [String](1) {}
        Dim listRow As ListViewItem
        Dim count As Integer = info.Lists.EmployeeEvents.Count
        For i As Integer = 0 To count - 1
            listData(0) = info.Lists.EmployeeEvents(i).EmployeeNumber
            listData(1) = info.Lists.EmployeeEvents(i).EmployeeEventCount.ToString()
            listRow = New ListViewItem(listData)
            lvwEmployeeTotals.Items.Add(listRow)
        Next

        ' Move the values from the Working List to the listView control using a
        ' foreach loop. The field names in the Events list have not been renamed
        ' in the LANSA Model to provide a contrast to the EmployeeEvents list.
        listData = New [String](3) {}
        For Each eventItem In info.Lists.Events
            listData(0) = eventItem.EMPNO
            listData(1) = eventItem.EMPEVTNUM.ToString()
            listData(2) = eventItem.EMPEVTDAT.ToShortDateString()
            listData(3) = eventItem.EMPEVTNTE
            listRow = New ListViewItem(listData)
            lvwEvents.Items.Add(listRow)
        Next

        lblTotalEntries.Text = [String].Concat("Total Entries: ", info.Out.TotalEvents.ToString())
        lblTimestamp.Text = [String].Concat("Queried at ", info.Out.TimeStamp.ToString())
    End Sub

    Private Sub PopulateEventsTable()
        Dim eventTypes As [String]() = {"AWD", "NOT", "PRO", "WRN"}

        Dim eventContext = New DataContext(masterContext, True)
        Dim evnt As [Event]

        ' Retrieve the list of employees as the employee associated with the event
        ' must be a valid employee.
        Dim columns = EmployeeCols.EmployeeNo + EmployeeCols.FirstName + EmployeeCols.Surname
        Dim employees = masterContext.Employees.RetrieveList(columns)

        Dim totalEvents As Short = 1
        For Each employee In employees
            For i As Short = 0 To totalEvents - 1
                ' Can't use "event" as a variable name as its a C# keyword
                evnt = New [Event]()

                evnt.EMPNO = employee.EmployeeNo
                evnt.EMPEVTNUM = i + 999990
                evnt.EMPEVTDAT = DateTime.Now
                evnt.EMPEVTTIM = DateTime.Now
                evnt.EMPEVTTYP = eventTypes(i)
                evnt.EMPEVTNTE = "This record has been inserted by the LANSA Open.Net sample application"
                evnt.EMPEVTUPD = i * 50
                evnt.EMPEVTUDT = DateTime.Now
                evnt.EMPEVTDOC = Nothing

                eventContext.Attach(evnt)
            Next

            If totalEvents < 4 Then
                totalEvents += 1
            Else
                totalEvents = 1
            End If
        Next

        Try
            eventContext.SubmitChanges()
        Catch ex As ApplyChangesException
            ' Handle conflicts but ignore other errors for this test.
            For Each changeConflict In ex.ChangeConflicts
                changeConflict.Resolve(ConflictResolveMode.OverwriteCurrentValues)
            Next
        Catch ex As System.Exception
            MessageBox.Show(ex.Message, "Error populating PSLEVENT")
        End Try
    End Sub
End Class
