﻿Public Class MainForm
    Private context1 As DataContext = Nothing
    Private context2 As DataContext = Nothing

    Private emp1 As Employee = Nothing
    Private emp2 As Employee = Nothing

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Create 2 contexts to simulate 2 different users
        Using cf As New Util.ConnectionForm
            cf.ShowDialog()

            ' Create context 1 (= user 1)
            context1 = New DataContext(True)
            context1.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password)

            ' Set UpdateModifiedColumnsOnly to false to ensure that all columns will be checked
            ' for update-conflicts when attempting to change their values.
            ' If UpdateModifiedColumnsOnly is false, only columns whose in-memory (local) values have been changed
            ' will be checked for update-conflict.
            context1.Options.UpdateModifiedColumnsOnly = False

            ' Create context 2 (= user 2)
            context2 = New DataContext(True)
            context2.Options.UpdateModifiedColumnsOnly = False
            context2.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password)

        End Using

        PopulateInstances(0)
    End Sub

    ' Retrieve employee details from the server
    Private Sub PopulateInstances(ByVal instance As Integer)
        Const empNo As String = "A1015"

        ' User 1
        If instance = 0 OrElse instance = 1 Then
            emp1 = context1.Employees.RetrieveItem(empNo)
            If emp1 IsNot Nothing Then
                boxEmpNo1.Text = emp1.EmployeeNo
                boxFirstName1.Text = emp1.FirstName
                boxSurname1.Text = emp1.Surname
                boxSalary1.Value = emp1.Salary
            End If
        End If

        ' User 2
        If instance = 0 OrElse instance = 2 Then
            emp2 = context2.Employees.RetrieveItem(empNo)
            If emp2 IsNot Nothing Then
                boxEmpNo2.Text = emp2.EmployeeNo
                boxFirstName2.Text = emp2.FirstName
                boxSurname2.Text = emp2.Surname
                boxSalary2.Value = emp2.Salary
            End If
        End If

        '
        If emp1 Is Nothing OrElse emp2 Is Nothing Then
            MessageBox.Show("Employee " & empNo & " does not exist", "Error")
        End If
    End Sub

    ' Save employee details of User 1
    Private Sub btnSave1_Click(ByVal sender As Object, ByVal e As EventArgs)
        If emp1 IsNot Nothing Then
            emp1.FirstName = boxFirstName1.Text
            emp1.Surname = boxSurname1.Text
            emp1.Salary = boxSalary1.Value

            SubmitChanges(context1)
            PopulateInstances(1)
        End If
    End Sub

    ' Save employee details of User 2
    Private Sub btnSave2_Click(ByVal sender As Object, ByVal e As EventArgs)
        If emp2 IsNot Nothing Then
            emp2.FirstName = boxFirstName2.Text
            emp2.Surname = boxSurname2.Text
            emp2.Salary = boxSalary2.Value

            SubmitChanges(context2)
            PopulateInstances(2)
        End If
    End Sub

    ' Apply changes to the server
    Private Sub SubmitChanges(ByVal context As DataContext)
        Dim repeat As Boolean
        Do
            repeat = False

            '
            Try
                context.SubmitChanges()
                MessageBox.Show("Update successful!")
            Catch ex As ApplyChangesException
                ' Check if update-conflicts have occurred
                If ex.ChangeConflicts.Count > 0 Then
                    ' Display the conflict-resolution form
                    Using form As New ConflictForm(ex)
                        Dim result As DialogResult = form.ShowDialog()
                        If result = DialogResult.OK Then
                            repeat = True
                        End If
                    End Using
                Else
                    MessageBox.Show(ex.Message)
                End If
            End Try
        Loop While repeat
    End Sub
End Class