﻿Public Class MainForm

    Private context As DataContext

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Connect to server
        context = New DataContext(False)

        Using cf As New Util.ConnectionForm
            cf.ShowDialog()
            context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password)
        End Using

        ' Populate tables drop down
        PopulateTables()
    End Sub

    Private Sub PopulateTables()
        For Each table In context.GetAllTables()
            Me.cbTables.Items.Add(New ListItem(table, table.Name))
        Next
    End Sub

    Private Function GetSelectedTable() As ITable
        Dim result As ITable = Nothing

        '
        If Me.cbTables.SelectedItem IsNot Nothing Then
            Dim listItem = TryCast(Me.cbTables.SelectedItem, ListItem)
            result = TryCast(listItem.AssociatedObject, ITable)
        End If

        '
        Return result
    End Function

    Private Sub cbTables_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles cbTables.SelectedIndexChanged
        Dim selectedTable = GetSelectedTable()

        '
        Me.lstColumns.Items.Clear()
        If selectedTable IsNot Nothing Then
            Dim columns = selectedTable.GetAllColumns()
            For Each column In columns
                Dim text As String = column.Description & " (" & column.Name & ")"
                Me.lstColumns.Items.Add(New ListItem(column, text), True)
            Next
        End If
    End Sub

    Private Sub btnRetrieveList_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnRetrieveList.Click
        Dim selectedTable = GetSelectedTable()

        '
        If selectedTable Is Nothing Then
            MessageBox.Show("There is no table selected", "Error")
        Else
            Dim columns = New List(Of Column)()

            '
            dataGrid.Columns.Clear()
            dataGrid.AutoGenerateColumns = False

            '
            For Each listItem As ListItem In Me.lstColumns.CheckedItems
                Dim column = TryCast(listItem.AssociatedObject, Column)
                columns.Add(column)

                '
                Dim dgColumn = New DataGridViewColumn()
                dgColumn.CellTemplate = New DataGridViewTextBoxCell()
                dgColumn.HeaderText = column.Description
                dgColumn.DataPropertyName = column.PropertyName
                dataGrid.Columns.Add(dgColumn)
            Next

            '
            Dim cond As String = Me.txtCondition.Text.Trim().ToUpperInvariant()

            '
            Try
                Dim list = selectedTable.RetrieveList(columns.ToArray(), cond)
                dataGrid.DataSource = list
            Catch ex As Exception
                MessageBox.Show(ex.Message, "Error")
            End Try
        End If
    End Sub

    Private Class ListItem
        Public ReadOnly AssociatedObject As Object
        Private ReadOnly text As String

        Public Sub New(ByVal associatedObject As Object, ByVal text As String)
            Me.AssociatedObject = associatedObject
            Me.text = text
        End Sub

        Public Overrides Function ToString() As String
            Return Me.text
        End Function
    End Class
End Class
