'*
' * This sample illustrates how to get list of spool files
' * and retrieve their contents.
' * 
' * Key methods:
' *   DataContext.Ibm.GetSpoolFileListForUser
' *   DataContext.Ibm.GetSpoolFileListForQueue
' *

Partial Public Class MainForm
    Inherits Form
    Private context As DataContext

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        cbType.SelectedIndex = 0

        '
        context = New DataContext(False)
        Using cf As ConnectionForm = New ConnectionForm
            cf.ShowDialog()
            context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password)
        End Using
    End Sub

    Private Sub btnGenerateList_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnGenerateList.Click
        lblStatus.Text = "Retrieving spool files ..."
        statusStrip.Refresh()
        Try
            Dim spoolFiles As SpoolFile()
            Dim name As String = txtName.Text.Trim().ToUpperInvariant()

            '
            If cbType.SelectedIndex = 0 Then
                ' Get spool file list for the specified user
                spoolFiles = context.Ibm.GetSpoolFileListForUser(name)
            Else
                ' Get spool file list for the specified queue
                spoolFiles = context.Ibm.GetSpoolFileListForQueue(name)
            End If

            '
            For Each spoolFile In spoolFiles
                Dim item As New ListViewItem()
                item.Text = spoolFile.Name
                item.SubItems.Add(spoolFile.UserName)
                item.SubItems.Add(spoolFile.Queue)
                item.SubItems.Add(spoolFile.JobName)
                item.SubItems.Add(spoolFile.JobNumber.ToString())
                item.SubItems.Add(spoolFile.Library)
                item.SubItems.Add(spoolFile.PageCount.ToString())
                item.SubItems.Add(spoolFile.CurrentPage.ToString())
                item.SubItems.Add(spoolFile.Priority)
                item.Tag = spoolFile

                '
                spoolFileList.Items.Add(item)
            Next
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        Finally
            lblStatus.Text = ""
        End Try
    End Sub

    Private Sub ViewSpoolFileContent()
        If spoolFileList.SelectedItems.Count > 0 Then
            Dim spoolFile As SpoolFile = TryCast(spoolFileList.SelectedItems(0).Tag, SpoolFile)
            If spoolFile IsNot Nothing Then
                lblStatus.Text = "Retrieving content ..."
                statusStrip.Refresh()
                Try
                    txtContent.Text = spoolFile.Content
                Catch ex As Exception
                    MessageBox.Show(ex.Message, "Error")
                Finally
                    lblStatus.Text = ""
                End Try
            End If
        Else
            MessageBox.Show("Please select a spool file to view", "Error")
        End If
    End Sub

    Private Sub btnViewContent_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnViewContent.Click
        ViewSpoolFileContent()
    End Sub

End Class
