using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Linq;
using System.Xml.Linq;

using Model;
using LOpen.EntityModel;

namespace Tutorials
{
	public partial class Tutorial3
	{
		DataContext MasterContext = new DataContext(false);

		public Tutorial3()
		{
			InitializeComponent();

			DataGridView1.AutoGenerateColumns = false;
		}

		private void Connect()
		{
			// Specify most details in the Settings object
			MasterContext.Settings.Server = "MyServer";
			MasterContext.Settings.Partition = "DEM";
			MasterContext.Settings.PortNumber = 4545;

			// Connect
			try
			{
				MasterContext.Connect("MyUsername", "MyPassword");
				MessageBox.Show("Connected to the Server.", "Connection", MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly, false);
			}
			catch(Exception ex)
			{
				MessageBox.Show("Error when trying to connect: " + ex.Message, "Connection", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign, false);
			}

			//
			btnConnect.Enabled = false;
		}

		private void btnConnect_Click(System.Object sender, System.EventArgs e)
		{
			Connect();
		}

		private void btnRetrieveAllRecords_Click(System.Object sender, System.EventArgs e)
		{
			var personList = MasterContext.Persons.RetrieveList();

			// Assign the results to DataGridView1
			DataGridView1.DataSource = personList;
		}

		private void btnRetrieveSomeColumns_Click(System.Object sender, System.EventArgs e)
		{
			var columns = PersonCols.GIVENAME + PersonCols.SURNAME;
			var personList = MasterContext.Persons.RetrieveList(columns);
		
			// Assign the results to DataGridView1
			DataGridView1.DataSource = personList;
		}

		private void btnRetrieveCondition1_Click(System.Object sender, System.EventArgs e)
		{
			var condition = PersonExprs.SALARY > 50000.00;
			var personList = MasterContext.Persons.RetrieveList(condition);
		
			// Assign the results to DataGridView1
			DataGridView1.DataSource = personList;
		}

		private void btnRetrieveCondition2_Click(System.Object sender, System.EventArgs e)
		{
			var condition = (PersonExprs.SALARY > 50000.00).And(PersonExprs.SALARY < 100000.00);
			var personList = MasterContext.Persons.RetrieveList(condition);

			// Assign the results to DataGridView1
			DataGridView1.DataSource = personList;
		}

		private void btnRetrieveExactMatch_Click(System.Object sender, System.EventArgs e)
		{
			var personList = MasterContext.Persons.RetrieveList(true, "A0070");

			// Assign the results to DataGridView1
			DataGridView1.DataSource = personList;
		}

		private void btnRetrievePartialMatch_Click(System.Object sender, System.EventArgs e)
		{
			var personList = MasterContext.Persons.RetrieveList(false, "A1");

			// Assign the results to DataGridView1
			DataGridView1.DataSource = personList;
		}

		private void btnRetrieveLogicalTable_Click(System.Object sender, System.EventArgs e)
		{
			var personList = MasterContext.Persons_by_SNameGName.RetrieveList(false, "S");

			// Assign the results to DataGridView1
			DataGridView1.DataSource = personList;
		}
	}
}