using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Linq;
using System.Xml.Linq;
using System.Text;

using Model;
using LOpen.EntityModel;

namespace Tutorials
{
	partial class Tutorial4
	{
		private DataContext MasterContext = new DataContext(false);

		// Context to be used to update/add/delete records
		private DataContext ChildContext = null;

		// Current person
		private Person CurrentPerson = null;

		public Tutorial4()
		{
			InitializeComponent();
		}

		private void Connect()
		{
			// Specify most details in the Settings object
			MasterContext.Settings.Server = "MyServer";
			MasterContext.Settings.Partition = "DEM";
			MasterContext.Settings.PortNumber = 4545;

			// Connect
			try
			{
				MasterContext.Connect("MyUsername", "MyPassword");
				MessageBox.Show("Connected to the Server.");
			}
			catch (System.Exception exc)
			{
				MessageBox.Show("Error when trying to connect: " + exc.Message, "Connection", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign, false);
			}

			//
			btnConnect.Enabled = false;
		}

		private void btnRetrieve_Click(System.Object sender, System.EventArgs e)
		{
			ChildContext = new DataContext(MasterContext, true);

			string empNo = boxSearch.Text.ToUpper();
			Person person = ChildContext.Persons.RetrieveItem(empNo);

			CurrentPerson = person;

			if (person != null)
			{
				// Assign the results to textboxes
				boxEmpNo.Text = CurrentPerson.EMPNO;
				boxSurname.Text = CurrentPerson.SURNAME;
				boxGivenName.Text = CurrentPerson.GIVENAME;
				boxStreet.Text = CurrentPerson.ADDRESS1;
				boxSuburb.Text = CurrentPerson.ADDRESS2;
				boxState.Text = CurrentPerson.ADDRESS3;
				boxPostCode.Value = CurrentPerson.POSTCODE;
				boxHomePhone.Text = CurrentPerson.PHONEHME;
				boxBusinessPhone.Text = CurrentPerson.PHONEBUS;
				boxStartDate.Value = CurrentPerson.STARTDTE;
				boxTerminationDate.Value = CurrentPerson.TERMDATE;
				boxDeptCode.Text = CurrentPerson.DEPTMENT;
				boxSectionCode.Text = CurrentPerson.SECTION;
				boxSalary.Value = CurrentPerson.SALARY;

				MessageBox.Show("Employee " + CurrentPerson.EMPNO + " retrieved.");
			}
			else
			{
				MessageBox.Show("Employee not found.");
			}
		}

		private void btnConnect_Click(System.Object sender, System.EventArgs e)
		{
			Connect();
		}

		private void btnUpdate_Click(System.Object sender, System.EventArgs e)
		{
			if (CurrentPerson != null)
			{
				// Get the person's details
				CurrentPerson.EMPNO = boxEmpNo.Text;
				CurrentPerson.SURNAME = boxSurname.Text;
				CurrentPerson.GIVENAME = boxGivenName.Text;
				CurrentPerson.ADDRESS1 = boxStreet.Text;
				CurrentPerson.ADDRESS2 = boxSuburb.Text;
				CurrentPerson.ADDRESS3 = boxState.Text;
				CurrentPerson.POSTCODE = (long)boxPostCode.Value;
				CurrentPerson.PHONEHME = boxHomePhone.Text;
				CurrentPerson.PHONEBUS = boxBusinessPhone.Text;
				CurrentPerson.STARTDTE = (long)boxStartDate.Value;
				CurrentPerson.TERMDATE = (long)boxTerminationDate.Value;
				CurrentPerson.DEPTMENT = boxDeptCode.Text;
				CurrentPerson.SECTION = boxSectionCode.Text;
				CurrentPerson.SALARY = boxSalary.Value;
				try
				{
					//Submit the changes
					ChildContext.SubmitChanges();

					MessageBox.Show("Employee " + CurrentPerson.EMPNO + " updated.");
				}
				catch (ApplyChangesException exception)
				{
					StringBuilder messageBuilder = new StringBuilder();
					foreach (DataObjectErrorInfo Error in exception.Errors)
					{
						string message = Error.Message;
						messageBuilder.AppendLine(message);
					}
					MessageBox.Show(messageBuilder.ToString());
				}
			}
			else
			{
				MessageBox.Show("You need to retrieve a person first before you can update anything.");
			}
		}

		private void btnAdd_Click(System.Object sender, System.EventArgs e)
		{
			ChildContext = new DataContext(MasterContext, true);

			//Create a new person data object
			Person person = new Person();

			//Get the properties of the person data object
			person.EMPNO = boxEmpNo.Text;
			person.SURNAME = boxSurname.Text;
			person.GIVENAME = boxGivenName.Text;
			person.ADDRESS1 = boxStreet.Text;
			person.ADDRESS2 = boxSuburb.Text;
			person.ADDRESS3 = boxState.Text;
			person.POSTCODE = (long)boxPostCode.Value;
			person.PHONEHME = boxHomePhone.Text;
			person.PHONEBUS = boxBusinessPhone.Text;
			person.STARTDTE = (long)boxStartDate.Value;
			person.TERMDATE = (long)boxTerminationDate.Value;
			person.DEPTMENT = boxDeptCode.Text;
			person.SECTION = boxSectionCode.Text;
			person.SALARY = boxSalary.Value;

			//Attach the new person to the context
			ChildContext.Attach(person);

			try
			{
				//Create a new person record on the server
				ChildContext.SubmitChanges();
				CurrentPerson = person;


				MessageBox.Show("Employee " + person.EMPNO + " added.");
			}
			catch (ApplyChangesException exception)
			{
				StringBuilder messageBuilder = new StringBuilder();
				foreach (DataObjectErrorInfo Error in exception.Errors)
				{
					string message = Error.Message;
					messageBuilder.AppendLine(message);
				}
				MessageBox.Show(messageBuilder.ToString());

				//
				CurrentPerson = null;
				ChildContext = null;

			}
		}

		private void  btnDelete_Click(System.Object sender, System.EventArgs e)
		{
			if (CurrentPerson != null)
			{
				//Mark the current person object for deletion
				ChildContext.Delete(CurrentPerson);

				try
				{
					// Ask the runtime to automatically delete dependent skill records ...
					ChildContext.Options.AutoDeleteDependentRecords = true;
					ChildContext.Options.PerformServerValidationOnSubmitChanges = false;

					// ... Or delete manually
					// ChildContext.PersonSkills.DirectDelete(CurrentPerson.EMPNO);

					//Delete the record on the server
					ChildContext.SubmitChanges();
					MessageBox.Show("Employee " + CurrentPerson.EMPNO + " deleted.");

					// Set the CurrentPerson to Nothing
					CurrentPerson = null;
					ChildContext = null;

					// Clear textboxes
					boxEmpNo.Text = "";
					boxSurname.Text = "";
					boxGivenName.Text = "";
					boxStreet.Text = "";
					boxSuburb.Text = "";
					boxState.Text = "";
					boxPostCode.Value = 0;
					boxHomePhone.Text = "";
					boxBusinessPhone.Text = "";
					boxStartDate.Value = 0;
					boxTerminationDate.Value = 0;
					boxDeptCode.Text = "";
					boxSectionCode.Text = "";
					boxSalary.Value = 0;
				}
				catch (ApplyChangesException exception)
				{
					StringBuilder messageBuilder = new StringBuilder();
					foreach (DataObjectErrorInfo Error in exception.Errors)
					{
						string message = Error.Message;
						messageBuilder.AppendLine(message);
					}
					MessageBox.Show(messageBuilder.ToString());
				}
			}
			else
			{
				MessageBox.Show("You need to retrieve a person first before you can delete anything.");
			}
		}
	}

}