using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Linq;
using System.Xml.Linq;
using System.Text;

using Model;
using LOpen.EntityModel;

namespace Tutorials
{
	public partial class Tutorial9
	{
		private DataContext MasterContext = new DataContext(false);

		// Context to be used to update/add/delete records
		private DataContext ChildContext = null;

		// Current person
		private Person CurrentPerson = null;

		public Tutorial9()
		{
			InitializeComponent();

			//
			DataContext.Events.LockObjectForUpdate += Lock;
			DataContext.Events.UnlockObjectAfterUpdate += ReleaseLock;
		}

		private void Connect()
		{
			MasterContext.Settings.Server = "MyServer";
			MasterContext.Settings.Partition = "DEM";
			MasterContext.Settings.PortNumber = 4545;

			if (boxLanguage.Text == "French")
			{
				MasterContext.Settings.Language = Language.FRA;
			}
			else
			{
				MasterContext.Settings.Language = Language.ENG;
			}

			// Set labels for the selected language
			SetDescriptions();

			// Connect
			try
			{
				MasterContext.Connect("MyUsername", "MyPassword");
				MessageBox.Show("Connected to the Server.");
			}
			catch (System.Exception exc)
			{
				MessageBox.Show("Error when trying to connect: " + exc.Message, "Connection", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign, false);
			}

			//
			btnConnect.Enabled = false;
		}

		private void Lock(object sender, LockObjectEventArgs args)
		{
			if (args.Object is Person)
			{
				Person Person = (Person)args.Object;
				try
				{
					MasterContext.LogicalLocks.PersonTableLock.Lock(Person.EMPNO);
					args.Successful = true;
					MessageBox.Show("Record successfully locked", "Locking");
				}
				catch (ObjectAlreadyLockedException)
				{
					args.Successful = false;
				}
			}
		}

		private void ReleaseLock(object sender, UnlockObjectEventArgs args)
		{
			if (args.Object is Person)
			{
				Person Person = (Person)args.Object;
				try
				{
					MasterContext.LogicalLocks.PersonTableLock.Unlock(Person.EMPNO);
					args.Successful = true;
					MessageBox.Show("Record successfully unlocked", "Locking");
				}
				catch (ObjectUnlockFailException)
				{
					args.Successful = false;
				}
			}
		}

		private void  btnRetrieve_Click(System.Object sender, System.EventArgs e)
		{
			ChildContext = new DataContext(MasterContext, true);

			string empNo = boxSearch.Text.ToUpper();
			Person person = ChildContext.Persons.RetrieveItem(empNo);

			CurrentPerson = person;

			if (person != null)
			{
				// Assign the results to textboxes
				boxEmpNo.Text = CurrentPerson.EMPNO;
				boxSurname.Text = CurrentPerson.SURNAME;
				boxGivenName.Text = CurrentPerson.GIVENAME;
				boxStreet.Text = CurrentPerson.ADDRESS1;
				boxSuburb.Text = CurrentPerson.ADDRESS2;
				boxState.Text = CurrentPerson.ADDRESS3;
				boxPostCode.Value = CurrentPerson.POSTCODE;
				boxHomePhone.Text = CurrentPerson.PHONEHME;
				boxBusinessPhone.Text = CurrentPerson.PHONEBUS;
				boxStartDate.Value = CurrentPerson.STARTDTE;
				boxTerminationDate.Value = CurrentPerson.TERMDATE;
				boxDeptCode.Text = CurrentPerson.DEPTMENT;
				boxSectionCode.Text = CurrentPerson.SECTION;
				boxSalary.Value = CurrentPerson.SALARY;

				MessageBox.Show("Employee " + CurrentPerson.EMPNO + " retrieved.");
			}
			else
			{
				MessageBox.Show("Employee not found.");
			}
		}

		private void btnConnect_Click(System.Object sender, System.EventArgs e)
		{
			Connect();
		}

		private void btnUpdate_Click(System.Object sender, System.EventArgs e)
		{
			if (CurrentPerson != null)
			{
				// Get the person's details
				CurrentPerson.EMPNO = boxEmpNo.Text;
				CurrentPerson.SURNAME = boxSurname.Text;
				CurrentPerson.GIVENAME = boxGivenName.Text;
				CurrentPerson.ADDRESS1 = boxStreet.Text;
				CurrentPerson.ADDRESS2 = boxSuburb.Text;
				CurrentPerson.ADDRESS3 = boxState.Text;
				CurrentPerson.POSTCODE = (long)boxPostCode.Value;
				CurrentPerson.PHONEHME = boxHomePhone.Text;
				CurrentPerson.PHONEBUS = boxBusinessPhone.Text;
				CurrentPerson.STARTDTE = (long)boxStartDate.Value;
				CurrentPerson.TERMDATE = (long)boxTerminationDate.Value;
				CurrentPerson.DEPTMENT = boxDeptCode.Text;
				CurrentPerson.SECTION = boxSectionCode.Text;
				CurrentPerson.SALARY = boxSalary.Value;

				MessageBox.Show("Employee in-memory data object " + CurrentPerson.EMPNO + " has been updated.");
				MessageBox.Show("Data object state is now " + ChildContext.GetObjectState(CurrentPerson).ToString());
			}
			else
			{
				MessageBox.Show("You need to retrieve a person first before you can update anything.");
			}
		}

		private void btnAdd_Click(System.Object sender, System.EventArgs e)
		{
			ChildContext = new DataContext(MasterContext, true);

			//Create a new person data object
			Person person = new Person();

			//Get the properties of the person data object
			person.EMPNO = boxEmpNo.Text;
			person.SURNAME = boxSurname.Text;
			person.GIVENAME = boxGivenName.Text;
			person.ADDRESS1 = boxStreet.Text;
			person.ADDRESS2 = boxSuburb.Text;
			person.ADDRESS3 = boxState.Text;
			person.POSTCODE = (long)boxPostCode.Value;
			person.PHONEHME = boxHomePhone.Text;
			person.PHONEBUS = boxBusinessPhone.Text;
			person.STARTDTE = (long)boxStartDate.Value;
			person.TERMDATE = (long)boxTerminationDate.Value;
			person.DEPTMENT = boxDeptCode.Text;
			person.SECTION = boxSectionCode.Text;
			person.SALARY = boxSalary.Value;

			//Attach the new person to the context
			ChildContext.Attach(person);
			CurrentPerson = person;

			MessageBox.Show("New employee data object has been created in the in-memory data context.");
			MessageBox.Show("Data object state is now " + ChildContext.GetObjectState(CurrentPerson).ToString());
		}

		private void btnDelete_Click(System.Object sender, System.EventArgs e)
		{
			if (CurrentPerson != null)
			{
				//Mark the current person object for deletion
				ChildContext.Delete(CurrentPerson);

				MessageBox.Show("Employee data object " + CurrentPerson.EMPNO + " has been marked for deletion.");
				MessageBox.Show("Data object state is now " + ChildContext.GetObjectState(CurrentPerson).ToString());
			}
			else
			{
				MessageBox.Show("You need to retrieve a person first before you can delete anything.");
			}
		}

		private void btnSubmitChanges_Click(System.Object sender, System.EventArgs e)
		{
			if (ChildContext != null)
			{
				try
				{
					if (ChildContext.GetObjectState(CurrentPerson) == DataObjectState.Deleted)
					{
						// Ask the runtime to delete dependent skill records automatically ...
						ChildContext.Options.AutoDeleteDependentRecords = true;

						ChildContext.Options.PerformServerValidationOnSubmitChanges = false;
					}
					// ... Or delete manually using 'DirectDelete' to delete multiple records.
					// MasterContext.PersonSkills.DirectDelete(PersonSkillExprs.EMPNO = TextBox1.Text.ToUpper())

					// Submit the changes
					ChildContext.SubmitChanges();

					//
					MessageBox.Show("Changes to employee " + CurrentPerson.EMPNO + " has been committed to the database");
					MessageBox.Show("Data object state is now " + ChildContext.GetObjectState(CurrentPerson).ToString());

					// If current person has been deleted ...
					if (ChildContext.GetObjectState(CurrentPerson) == DataObjectState.Detached)
					{
						CurrentPerson = null;
						ChildContext = null;

						//
						boxEmpNo.Text = "";
						boxSurname.Text = "";
						boxGivenName.Text = "";
						boxStreet.Text = "";
						boxSuburb.Text = "";
						boxState.Text = "";
						boxPostCode.Value = 0;
						boxHomePhone.Text = "";
						boxBusinessPhone.Text = "";
						boxStartDate.Value = 0;
						boxTerminationDate.Value = 0;
						boxDeptCode.Text = "";
						boxSectionCode.Text = "";
						boxSalary.Value = 0;

					}
				}
				catch (ApplyChangesException exception)
				{
					StringBuilder messageBuilder = new StringBuilder();
					foreach (var ErrorItem in exception.Errors)
					{
						messageBuilder.AppendLine(ErrorItem.Message);
					}
					MessageBox.Show(messageBuilder.ToString());

				}
			}
			else
			{
				MessageBox.Show("There is nothing to commit.");
			}
		}

		private void btnValidate_Click(System.Object sender, System.EventArgs e)
		{
			if (ChildContext != null)
			{
				DataValidationResult result = ChildContext.ValidateChanges();

				// Check if validation is successful or not?
				if (!result.IsSuccessful)
				{
					// "InvalidObjects" property gives us the list of each data object that fails the validation.
					foreach (var errorInfo in result.InvalidObjects)
					{
						// The 'Columns' property of the error info tells us which columns
						// contain invalid values
						string message = "The following fields contain invalid values: ";
						foreach (Column column in errorInfo.Columns)
						{
							message += "\\n" + column.Label;
						}

						// The 'Errors' property gives us the list of validation messages
						message += "Error messages:";
						foreach (Error error in errorInfo.Errors)
						{
							message += "\\n" + error.Message;
						}
						MessageBox.Show(message);
					}
				}
				else
				{
					MessageBox.Show("Validation was successful", "Validation", MessageBoxButtons.OK);
				}
			}
			else
			{
				MessageBox.Show("There is nothing to validate");
			}
		}

		public void SetDescriptions()
		{
			Label1.Text = MasterContext.Persons.Columns.EMPNO.Description;
			Label2.Text = MasterContext.Persons.Columns.SURNAME.Description;
			Label3.Text = MasterContext.Persons.Columns.GIVENAME.Description;
			Label4.Text = MasterContext.Persons.Columns.ADDRESS1.Description;
			Label5.Text = MasterContext.Persons.Columns.ADDRESS2.Description;
			Label6.Text = MasterContext.Persons.Columns.ADDRESS3.Description;
			Label7.Text = MasterContext.Persons.Columns.POSTCODE.Description;
			Label8.Text = MasterContext.Persons.Columns.PHONEHME.Description;
			Label9.Text = MasterContext.Persons.Columns.PHONEBUS.Description;
			Label10.Text = MasterContext.Persons.Columns.STARTDTER.Description;
			Label11.Text = MasterContext.Persons.Columns.TERMDATE.Description;
			Label12.Text = MasterContext.Persons.Columns.DEPTMENT.Description;
			Label13.Text = MasterContext.Persons.Columns.SECTION.Description;
			Label14.Text = MasterContext.Persons.Columns.SALARY.Description;

			btnDelete.Text = MasterContext.Localized._DEMDELETE;
		}
	}
}