﻿Imports System.Text
Imports Tutorials.Model

Public Class Tutorial4
    Private MasterContext As New DataContext(False)

    ' Context to be used to update/add/delete records
    Private ChildContext As DataContext = Nothing

    ' Current person
    Private CurrentPerson As Person = Nothing

    Private Sub Connect()
        ' Specify most details in the Settings object
        MasterContext.Settings.Server = "MyServer"
        MasterContext.Settings.Partition = "DEM"
        MasterContext.Settings.PortNumber = 4545

        ' Connect
        Try
            MasterContext.Connect("MyUsername", "MyPassword")
            MessageBox.Show("Connected to the Server.")
        Catch exc As System.Exception
            MessageBox.Show("Error when trying to connect: " & exc.Message, "Connection", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign, False)
        End Try

        '
        btnConnect.Enabled = False
    End Sub

    Private Sub btnRetrieve_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRetrieve.Click
        ChildContext = New DataContext(MasterContext, True)

        Dim empNo As String = boxSearch.Text.ToUpper()
        Dim person As Person = ChildContext.Persons.RetrieveItem(empNo)

        CurrentPerson = person

        If person IsNot Nothing Then
            ' Assign the results to textboxes
            boxEmpNo.Text = CurrentPerson.EMPNO
            boxSurname.Text = CurrentPerson.SURNAME
            boxGivenName.Text = CurrentPerson.GIVENAME
            boxStreet.Text = CurrentPerson.ADDRESS1
            boxSuburb.Text = CurrentPerson.ADDRESS2
            boxState.Text = CurrentPerson.ADDRESS3
            boxPostCode.Value = CurrentPerson.POSTCODE
            boxHomePhone.Text = CurrentPerson.PHONEHME
            boxBusinessPhone.Text = CurrentPerson.PHONEBUS
            boxStartDate.Value = CurrentPerson.STARTDTE
            boxTerminationDate.Value = CurrentPerson.TERMDATE
            boxDeptCode.Text = CurrentPerson.DEPTMENT
            boxSectionCode.Text = CurrentPerson.SECTION
            boxSalary.Value = CurrentPerson.SALARY

            MessageBox.Show("Employee " + CurrentPerson.EMPNO + " retrieved.")
        Else
            MessageBox.Show("Employee not found.")
        End If
    End Sub

    Private Sub btnConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click
        Call Connect()
    End Sub

    Private Sub btnUpdate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click
        If CurrentPerson IsNot Nothing Then
            ' Get the person's details
            CurrentPerson.EMPNO = boxEmpNo.Text
            CurrentPerson.SURNAME = boxSurname.Text
            CurrentPerson.GIVENAME = boxGivenName.Text
            CurrentPerson.ADDRESS1 = boxStreet.Text
            CurrentPerson.ADDRESS2 = boxSuburb.Text
            CurrentPerson.ADDRESS3 = boxState.Text
            CurrentPerson.POSTCODE = boxPostCode.Value
            CurrentPerson.PHONEHME = boxHomePhone.Text
            CurrentPerson.PHONEBUS = boxBusinessPhone.Text
            CurrentPerson.STARTDTE = boxStartDate.Value
            CurrentPerson.TERMDATE = boxTerminationDate.Value
            CurrentPerson.DEPTMENT = boxDeptCode.Text
            CurrentPerson.SECTION = boxSectionCode.Text
            CurrentPerson.SALARY = boxSalary.Value
            Try
                'Submit the changes
                ChildContext.SubmitChanges()
                MessageBox.Show("Employee " + CurrentPerson.EMPNO + " updated.")

            Catch exception As ApplyChangesException
                Dim messageBuilder As New StringBuilder
                For Each [Error] As DataObjectErrorInfo In exception.Errors
                    Dim message As String = [Error].Message
                    messageBuilder.AppendLine(message)
                Next
                MessageBox.Show(messageBuilder.ToString())
            End Try
        Else
            MessageBox.Show("You need to retrieve a person first before you can update anything.")
        End If
    End Sub

    Private Sub btnAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        ChildContext = New DataContext(MasterContext, True)

        'Create a new person data object
        Dim person As New Person

        'Get the properties of the person data object
        person.EMPNO = boxEmpNo.Text
        person.SURNAME = boxSurname.Text
        person.GIVENAME = boxGivenName.Text
        person.ADDRESS1 = boxStreet.Text
        person.ADDRESS2 = boxSuburb.Text
        person.ADDRESS3 = boxState.Text
        person.POSTCODE = boxPostCode.Value
        person.PHONEHME = boxHomePhone.Text
        person.PHONEBUS = boxBusinessPhone.Text
        person.STARTDTE = boxStartDate.Value
        person.TERMDATE = boxTerminationDate.Value
        person.DEPTMENT = boxDeptCode.Text
        person.SECTION = boxSectionCode.Text
        person.SALARY = boxSalary.Value

        'Attach the new person to the context
        ChildContext.Attach(person)

        Try
            'Create a new person record on the server
            ChildContext.SubmitChanges()
            CurrentPerson = person

            MessageBox.Show("Employee " + person.EMPNO + " added.")

        Catch exception As ApplyChangesException
            Dim messageBuilder As New StringBuilder
            For Each [Error] As DataObjectErrorInfo In exception.Errors
                Dim message As String = [Error].Message
                messageBuilder.AppendLine(message)
            Next
            MessageBox.Show(messageBuilder.ToString())

            '
            CurrentPerson = Nothing
            ChildContext = Nothing
        End Try

    End Sub

    Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button5.Click
        If CurrentPerson IsNot Nothing Then
            'Mark the current person object for deletion
            ChildContext.Delete(CurrentPerson)

            Try
                ' Ask the runtime to automatically delete dependent skill records ...
                ChildContext.Options.AutoDeleteDependentRecords = True
                ChildContext.Options.PerformServerValidationOnSubmitChanges = False

                ' ... Or delete manually
                ' ChildContext.PersonSkills.DirectDelete(CurrentPerson.EMPNO)

                'Delete the record on the server
                ChildContext.SubmitChanges()
                MessageBox.Show("Employee " + CurrentPerson.EMPNO + " deleted.")

                ' Set the CurrentPerson to Nothing
                CurrentPerson = Nothing
                ChildContext = Nothing

                ' Clear textboxes
                boxEmpNo.Text = ""
                boxSurname.Text = ""
                boxGivenName.Text = ""
                boxStreet.Text = ""
                boxSuburb.Text = ""
                boxState.Text = ""
                boxPostCode.Value = 0
                boxHomePhone.Text = ""
                boxBusinessPhone.Text = ""
                boxStartDate.Value = 0
                boxTerminationDate.Value = 0
                boxDeptCode.Text = ""
                boxSectionCode.Text = ""
                boxSalary.Value = 0
            Catch exception As ApplyChangesException
                Dim messageBuilder As New StringBuilder
                For Each [Error] As DataObjectErrorInfo In exception.Errors
                    Dim message As String = [Error].Message
                    messageBuilder.AppendLine(message)
                Next
                MessageBox.Show(messageBuilder.ToString())
            End Try
        Else
            MessageBox.Show("You need to retrieve a person first before you can delete anything.")
        End If
    End Sub
End Class

