﻿Imports System.Text
Imports Tutorials.Model

Public Class Tutorial6
    Private MasterContext As New DataContext(False)

    ' Context to be used to update/add/delete records
    Private ChildContext As DataContext = Nothing

    ' Current person
    Private CurrentPerson As Person = Nothing

    Private Sub Connect()
        MasterContext.Settings.Server = "MyServer"
        MasterContext.Settings.Partition = "DEM"
        MasterContext.Settings.PortNumber = 4545

        If boxLanguage.Text = "French" Then
            MasterContext.Settings.Language = Language.FRA
        Else
            MasterContext.Settings.Language = Language.ENG
        End If

        ' Set labels for the selected language
        SetDescriptions()

        ' Connect
        Try
            MasterContext.Connect("MyUsername", "MyPassword")
            MessageBox.Show("Connected to the Server.")
        Catch exc As System.Exception
            MessageBox.Show("Error when trying to connect: " & exc.Message, "Connection", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign, False)
        End Try

        '
        btnConnect.Enabled = False
    End Sub

    Private Sub btnRetrieve_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRetrieve.Click
        ChildContext = New DataContext(MasterContext, True)

        Dim empNo As String = boxSearch.Text.ToUpper()
        Dim person As Person = ChildContext.Persons.RetrieveItem(empNo)

        CurrentPerson = person

        If person IsNot Nothing Then
            ' Assign the results to textboxes
            boxEmpNo.Text = CurrentPerson.EMPNO
            boxSurname.Text = CurrentPerson.SURNAME
            boxGivenName.Text = CurrentPerson.GIVENAME
            boxStreet.Text = CurrentPerson.ADDRESS1
            boxSuburb.Text = CurrentPerson.ADDRESS2
            boxState.Text = CurrentPerson.ADDRESS3
            boxPostCode.Value = CurrentPerson.POSTCODE
            boxHomePhone.Text = CurrentPerson.PHONEHME
            boxBusinessPhone.Text = CurrentPerson.PHONEBUS
            boxStartDate.Value = CurrentPerson.STARTDTE
            boxTerminationDate.Value = CurrentPerson.TERMDATE
            boxDeptCode.Text = CurrentPerson.DEPTMENT
            boxSectionCode.Text = CurrentPerson.SECTION
            boxSalary.Value = CurrentPerson.SALARY

            MessageBox.Show("Employee " + CurrentPerson.EMPNO + " retrieved.")
        Else
            MessageBox.Show("Employee not found.")
        End If
    End Sub

    Private Sub btnConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click
        Connect()
    End Sub

    Private Sub btnUpdate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnUpdate.Click
        If CurrentPerson IsNot Nothing Then
            ' Get the person's details
            CurrentPerson.EMPNO = boxEmpNo.Text
            CurrentPerson.SURNAME = boxSurname.Text
            CurrentPerson.GIVENAME = boxGivenName.Text
            CurrentPerson.ADDRESS1 = boxStreet.Text
            CurrentPerson.ADDRESS2 = boxSuburb.Text
            CurrentPerson.ADDRESS3 = boxState.Text
            CurrentPerson.POSTCODE = boxPostCode.Value
            CurrentPerson.PHONEHME = boxHomePhone.Text
            CurrentPerson.PHONEBUS = boxBusinessPhone.Text
            CurrentPerson.STARTDTE = boxStartDate.Value
            CurrentPerson.TERMDATE = boxTerminationDate.Value
            CurrentPerson.DEPTMENT = boxDeptCode.Text
            CurrentPerson.SECTION = boxSectionCode.Text
            CurrentPerson.SALARY = boxSalary.Value

            MessageBox.Show("Employee in-memory data object " + CurrentPerson.EMPNO + " has been updated.")
            MessageBox.Show("Data object state is now " & ChildContext.GetObjectState(CurrentPerson).ToString())
        Else
            MessageBox.Show("You need to retrieve a person first before you can update anything.")
        End If
    End Sub

    Private Sub btnAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAdd.Click
        ChildContext = New DataContext(MasterContext, True)

        'Create a new person data object
        Dim person As New Person

        'Get the properties of the person data object
        person.EMPNO = boxEmpNo.Text
        person.SURNAME = boxSurname.Text
        person.GIVENAME = boxGivenName.Text
        person.ADDRESS1 = boxStreet.Text
        person.ADDRESS2 = boxSuburb.Text
        person.ADDRESS3 = boxState.Text
        person.POSTCODE = boxPostCode.Value
        person.PHONEHME = boxHomePhone.Text
        person.PHONEBUS = boxBusinessPhone.Text
        person.STARTDTE = boxStartDate.Value
        person.TERMDATE = boxTerminationDate.Value
        person.DEPTMENT = boxDeptCode.Text
        person.SECTION = boxSectionCode.Text
        person.SALARY = boxSalary.Value

        'Attach the new person to the context
        ChildContext.Attach(person)
        CurrentPerson = person

        MessageBox.Show("New employee data object has been created in the in-memory data context.")
        MessageBox.Show("Data object state is now " + ChildContext.GetObjectState(CurrentPerson).ToString())
    End Sub

    Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelete.Click
        If CurrentPerson IsNot Nothing Then
            'Mark the current person object for deletion
            ChildContext.Delete(CurrentPerson)

            MessageBox.Show("Employee data object " + CurrentPerson.EMPNO + " has been marked for deletion.")
            MessageBox.Show("Data object state is now " + ChildContext.GetObjectState(CurrentPerson).ToString())
        Else
            MessageBox.Show("You need to retrieve a person first before you can delete anything.")
        End If
    End Sub

    Private Sub btnSubmitChanges_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSubmitChanges.Click
        If ChildContext IsNot Nothing Then
            Try
                If ChildContext.GetObjectState(CurrentPerson) = DataObjectState.Deleted Then
                    ' Ask the runtime to delete dependent skill records automatically ...
                    ChildContext.Options.AutoDeleteDependentRecords = True
                    ChildContext.Options.PerformServerValidationOnSubmitChanges = False

                    ' ... Or delete manually using 'DirectDelete' to delete multiple records.
                    ' MasterContext.PersonSkills.DirectDelete(PersonSkillExprs.EMPNO = TextBox1.Text.ToUpper())
                End If

                ' Submit the changes
                ChildContext.SubmitChanges()

                '
                MessageBox.Show("Changes to employee " + CurrentPerson.EMPNO + " has been committed to the database")
                MessageBox.Show("Data object state is now " + ChildContext.GetObjectState(CurrentPerson).ToString())

                ' If current person has been deleted ...
                If ChildContext.GetObjectState(CurrentPerson) = DataObjectState.Detached Then
                    CurrentPerson = Nothing
                    ChildContext = Nothing

                    '
                    boxEmpNo.Text = ""
                    boxSurname.Text = ""
                    boxGivenName.Text = ""
                    boxStreet.Text = ""
                    boxSuburb.Text = ""
                    boxState.Text = ""
                    boxPostCode.Value = 0
                    boxHomePhone.Text = ""
                    boxBusinessPhone.Text = ""
                    boxStartDate.Value = 0
                    boxTerminationDate.Value = 0
                    boxDeptCode.Text = ""
                    boxSectionCode.Text = ""
                    boxSalary.Value = 0
                End If

            Catch exception As ApplyChangesException
                Dim messageBuilder As New StringBuilder
                For Each ErrorItem In exception.Errors
                    messageBuilder.AppendLine(ErrorItem.Message)
                Next
                MessageBox.Show(messageBuilder.ToString())
            End Try

        Else
            MessageBox.Show("There is nothing to commit.")
        End If
    End Sub

    Private Sub btnValidate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnValidate.Click
        If ChildContext IsNot Nothing Then
            Dim result As DataValidationResult = ChildContext.ValidateChanges()

            ' Check if validation is successful or not?
            If Not result.IsSuccessful Then
                ' "InvalidObjects" property gives us the list of each data object that fails the validation.
                For Each errorInfo In result.InvalidObjects
                    ' The 'Columns' property of the error info tells us which columns
                    ' contain invalid values
                    Dim message As String = "The following fields contain invalid values: "
                    For Each column As Column In errorInfo.Columns
                        message += "\n" + column.Label
                    Next

                    ' The 'Errors' property gives us the list of validation messages
                    message += "Error messages:"
                    For Each [error] As [Error] In errorInfo.Errors
                        message += "\n" + [error].Message
                    Next
                    MessageBox.Show(message)
                Next
            Else
                MessageBox.Show("Validation was successful", "Validation", MessageBoxButtons.OK)
            End If
        Else
            MessageBox.Show("There is nothing to validate")
        End If
    End Sub

    Sub SetDescriptions()
        Label1.Text = MasterContext.Persons.Columns.EMPNO.Description
        Label2.Text = MasterContext.Persons.Columns.SURNAME.Description
        Label3.Text = MasterContext.Persons.Columns.GIVENAME.Description
        Label4.Text = MasterContext.Persons.Columns.ADDRESS1.Description
        Label5.Text = MasterContext.Persons.Columns.ADDRESS2.Description
        Label6.Text = MasterContext.Persons.Columns.ADDRESS3.Description
        Label7.Text = MasterContext.Persons.Columns.POSTCODE.Description
        Label8.Text = MasterContext.Persons.Columns.PHONEHME.Description
        Label9.Text = MasterContext.Persons.Columns.PHONEBUS.Description
        Label10.Text = MasterContext.Persons.Columns.STARTDTER.Description
        Label11.Text = MasterContext.Persons.Columns.TERMDATE.Description
        Label12.Text = MasterContext.Persons.Columns.DEPTMENT.Description
        Label13.Text = MasterContext.Persons.Columns.SECTION.Description
        Label14.Text = MasterContext.Persons.Columns.SALARY.Description

        btnDelete.Text = MasterContext.Localized._DEMDELETE
    End Sub
End Class